@@ -11,27 +11,34 @@ use strict ;
 ## our (%german, %english);
 
 ## Also this declearation fails unfortunately :-(((
-my ( $sysconfdir,   # directory for system configuration
-     $configname,   # name of configuration file in this directory
-     $pcd_info,     # location of PCD info file on Kodak Photo CDs
-     $pcd2html_home,# location of user configuration
-     $config,       # configuration file
-     $pcd2html_uti
+my ( $sysconfdir,    # directory for system configuration
+     $configname,    # name of configuration file in this directory
+     $pcd_info,      # location of PCD info file on Kodak Photo CDs
+     $pcd2html_home, # location of user configuration
+     $pcd2html_source_url, # URL where to find information about pcd2html
+     $config,        # configuration file
+     $pcd2html_uti,  #
+     $footer_string, # Create string for footer only once
+     %available_keys # Dictionar with key -> directory
    );
 
 my ($cdrom) ;
 
 $sysconfdir    = "/etc" ;
 $configname    = "pcd2html.conf" ;
-$pcd_info      = "photo_cd/info.pcd" ;
+$pcd_info      = "info.pcd" ;
 $pcd2html_home = $ENV{"HOME"} . "/.pcd2html" ;
 
+$pcd2html_source_url = "http://fam-tille.de/debian/pcd2html.html" ;
+
+my $PCD2HTMLMARKER = "##PCD2HTMLMARKER##" ;
+
 $config        = "$sysconfdir/$configname";
 stat($config) || die "Missing config file $config" ; 
 open(CONF, $config) || die "Error while opening config file $config" ;
 while ( <CONF> ) {
-  unless ( /[ \t]*#/ ) {
-    chomp( $cdrom = $_ );   # path to cdrom optained from config file
+  unless ( /^\s*#/ ) {
+    chomp( $cdrom = $_ );   # path to cdrom (cdrom images) obtained from config file
   }
 }
 close(CONF) || die "Error while closing file $config" ;
@@ -73,6 +80,10 @@ sub GetInternalInfoKey {
   return $internal_info_key ;
 }
 
+sub GetPcd2HtmlSourceUrl {
+  return $pcd2html_source_url ;
+}
+
 # some definitions for image conversion
 my $JPEG_QUALITY = 70 ;
 sub GetQuality {
@@ -100,6 +111,7 @@ my %english = ( "hext",   ".html" ,
 		"back_m", "back to main page" ,
 		"top",    "back" ,
 		"home",   "Home page" ,
+		"oicon",  "deutsch.png" ,
 		"other",  "index_de.html"
 	      );
 # some German translations
@@ -115,25 +127,48 @@ my %german = (  "hext",  "_de.html" ,
 		"back_m", "zur&uuml;ck zur Hauptseite" ,
 		"top",    "zur&uuml;ck" ,
 		"home",   "Hauptseite" ,
+		"oicon",  "english.png" ,
 		"other",  "index.html"
 	     );
 
 sub GetConvert {
-    return "convert -verbose" ;
+    # Since ImageMagick 6.0.3 convert uses a new syntax ... that's why
+    # here only the options are provided and convert itself is insertet
+    # in the calling procedure
+    # return "convert -verbose" ;
+    return " -verbose" ;
 }
 
 sub GetIconConvert {
-    return GetConvert() . " -colors 64 -dither -geometry $IconWidth" . 'x' . "$IconHeight" ;
+    return GetConvert() . " -dither -geometry $IconWidth" . 'x' . "$IconHeight" ;
 }
 
 $pcd2html_uti = "pcd2html_uti" ;
 
-# check whether a CD is mounted and try to mount if fail
-# return true if the mounted CD is a Kodak Photo CD
-sub mounted_pcd {
-  my ( $ostype );
 
-  unless ( mounted() ) {
+# Return a list of potential PCD dirs
+sub get_pcd_dirs {
+
+    my @dirs = () ;
+    my $check_command = "find $cdrom -follow \\( -type d -or -type l \\) -and -name photo_cd" ;
+    open (FINDDIRS, "$check_command |") || die ( "Kann $check_command nicht ausfhren." ) ;
+    while ( <FINDDIRS> ) {
+	chomp ;
+	@dirs = (@dirs, $_) ;
+    }
+    close (FINDDIRS) ;
+    return @dirs ;
+}
+
+# check whether a PCD directories are available
+# try to mount if fail
+# return list of directories with PCD structures
+sub mounted_pcd {
+  my ( $ostype, $info, @pcd_dirs );
+
+  my @check_pcd_dirs = get_pcd_dirs ();
+
+  if ( ! @check_pcd_dirs ) { 
     $ostype = $ENV{"OSTYPE"} ;
     if ( ! $ostype ) {
       $ostype = `uname` ;
@@ -154,102 +189,112 @@ sub mounted_pcd {
       warn "Unable to mount $cdrom.  Please ask your system administrator." ;
       return 0;
     }
+    @check_pcd_dirs = get_pcd_dirs ();
+    if ( ! @check_pcd_dirs ) {
+	return () ;
+    }
   }
 
-  unless ( stat("$cdrom/$pcd_info") ) {
-    warn "CD in $cdrom doesn't seem to be a Kodak Photo CD!\n";
-    return 0;
+  foreach ( @check_pcd_dirs ) {
+      if ( -e "$_/$pcd_info" ) {
+	  s?^$cdrom/?? ;
+	  @pcd_dirs = (@pcd_dirs, $_) ;
+      } else {
+	  warn "Directory in $_ doesn't seem to contain a Kodak Photo CD!\n";
+      }
   }
-  return 1;
+  return @pcd_dirs;
 }
 
-# Check output of `mount` for CD dircetory
-sub mounted {
-  my $mounted = `mount`;
-
-  if ( $mounted =~ /$cdrom/ ) { 
-    return 1
-  }
-  $mounted = `ls $cdrom` ;
-
-  if ( $mounted =~ /photo_cd/ ) { 
-    return 1
-  }
-  return 0;
-}
-
-
-#### Method: pcd_get_key
+#### Method: pcd_get_keys
 # Obtain Kodak Photo CD keys
 #
 ####
-sub pcd_get_key {
-  my ( $key, $info, %keys ) ;
+sub pcd_get_keys {
+    my ( $key, $info, %keys, @pcd_dirs ) ;
 
-  unless ( mounted_pcd() ) {
-    warn "No Kodak Photo CD mounted on $cdrom.\n";
-    return undef ;
-  }
-  
-  unless ( open(INFO, "$cdrom/$pcd_info" ) ) {
-     warn "Error while opening Kodak Photo CD information\n" ;
-     return undef ;
-  }
-  chomp ($info = <INFO>) ;
-  close(INFO) ;
-  $info =~ s/\D*(\d*)\D.*/$1/ ;
-  
-  if ( stat( "$pcd2html_home" ) ) {
-    unless ( open(HOME, "$pcd2html_home" ) ) {
-      warn "Error while opening $pcd2html_home\n" ;
-      return undef ;
+    # do not obtain the keys more than once ...
+    if ( keys(%available_keys) ) { return %available_keys ; }
+
+    @pcd_dirs = mounted_pcd() ;
+    unless ( @pcd_dirs ) {
+	warn "No Kodak Photo images available at $cdrom.\n";
+	return undef ;
     }
-    while ( <HOME> ) {
-      unless ( /\s*#/ ) {
-        chomp ;
-        s/# .*// ;
-        $a = $b = $_ ;
-        $a =~ s/\s*(\w*)\s.*/$1/ ;
-        $b =~ s/\s*$a\s*(\w*)\s*.*/$1/ ;
-        $keys{$b} = $a ;
-      }
+
+    # Search used PCD database in ${HOME}
+    if ( stat( "$pcd2html_home" ) ) {
+	unless ( open(HOME, "$pcd2html_home" ) ) {
+	    warn "Error while opening $pcd2html_home\n" ;
+	    return undef ;
+	}
+	while ( <HOME> ) {
+	    unless ( /\s*#/ ) {
+		chomp ;
+		s/# .*// ;
+		$a = $b = $_ ;
+		$a =~ s/\s*(\w*)\s.*/$1/ ;
+		$b =~ s/\s*$a\s*(\w*)\s*.*/$1/ ;
+		$keys{$b} = $a ;
+	    }
+	}
+	close(HOME);
+    } else {
+	# Create new database otherwise
+	unless ( open(HOME, ">$pcd2html_home" ) ) {
+	    warn "Error while creating $pcd2html_home\n" ;
+	    return undef;
+	}
+	select (HOME) ;
+	print "# pcd2html: keys for different Kodak Photo CDs\n" ;
+	print "# <key used in rules file> <specification number of CD>\n";
+	print "#   or for other images\n";
+	print "# <key used in rules file> <directory where to find the image>\n";
+	print "#\n" ;
+	select (STDOUT) ;
+	close(HOME) ;
+    }  
+
+    my $i = 0 ;
+    foreach ( @pcd_dirs ) {
+	unless ( open(INFO, "$cdrom/$_/$pcd_info" ) ) {
+	    warn "Error while opening Kodak Photo CD information at $cdrom/$_/$pcd_info\n" ;
+	    return undef ;
+	}
+	chomp ($info = <INFO>) ;
+	close(INFO) ;
+	$info =~ s/\D*(\d*)\D.*/$1/ ;
+	unless ( $info ) {
+	    print STDERR "$_/$pcd_info does not seem to be a valid Photo CD structure. Ignored.\n" ;
+	    next ;
+	}
+
+	$key = $keys{$info} ;
+	$i  += 1 ;
+	$available_keys{$key} = $_ ;
+	
+	unless ( $key ) {
+	    print "This Kodak Photo CD was not used before.\n" ;
+	    print "Which key should be used in the rules file? " ;
+	    while ( 1 ) {
+		$key = <STDIN> ;
+		chomp ( $key ) ;
+		unless ( $key =~ /\W/ ) { last ; }
+		print "Only letters, digits and `_` are allowed in keys\n" ;
+		print "Please insert valid key! " ;
+	    }
+	    unless ( open(HOME, ">>$pcd2html_home" ) ) {
+		warn "Error while opening $pcd2html_home\n" ;
+		return undef ;
+	    }
+	    select (HOME) ;
+	    print "$key $info\n" ;
+	    select (STDOUT) ;
+	    close(HOME) ;
+	}
     }
-    close(HOME);
-    $key = $keys{$info} ;
-  } else {
-    unless ( open(HOME, ">$pcd2html_home" ) ) {
-      warn "Error while creating $pcd2html_home\n" ;
-      return undef;
-    }
-    select (HOME) ;
-    print "# pcd2html: keys for different Kodak Photo CDs\n" ;
-    print "# <key used in rules file> <specification number of CD>\n";
-    print "#   or for other images\n";
-    print "# <key used in rules file> <directory where to find the image>\n";
-    print "#\n" ;
-    select (STDOUT) ;
-    close(HOME) ;
-  }  
-  unless ( $key ) {
-    print "This Kodak Photo CD was not used before.\n" ;
-    print "Which key should be used in the rules file? " ;
-    while ( 1 ) {
-      $key = <STDIN> ;
-      chomp ( $key ) ;
-      unless ( $key =~ /\W/ ) { last ; }
-      print "Only letters, digits and `_` are allowed in keys\n" ;
-      print "Please insert valid key! " ;
-    }
-    unless ( open(HOME, ">>$pcd2html_home" ) ) {
-      warn "Error while opening $pcd2html_home\n" ;
-      return undef ;
-    }
-    select (HOME) ;
-    print "$key $info\n" ;
-    select (STDOUT) ;
-    close(HOME) ;
-  }
-  return $key ;
+
+    return %available_keys ;
 }  
 
 #### Method: pcd_get_dirs ################################
@@ -285,15 +330,16 @@ sub pcd_get_dirs {
 # open rules file of a subdriectory
 #
 sub OpenRules {
-  my ( $rules ) ;
+    my ( $rules, %keys ) ;
 
-  $rules = "rules" ;
+    $rules = "rules" ;
 
-  unless ( -f $rules ) { die "There is no $rules file.\n" ; }
+    unless ( -f $rules ) { die "There is no $rules file.\n" ; }
+    unless ( open(RULES, $rules ) ) { die "Error while opening $rules. ($!)\n" ; }
 
-  unless  ( pcd_get_key() ) { print STDERR "pcd2html: Warning: No PCD inserted.\n" ; }
-
-  unless ( open(RULES, $rules ) ) { die "Error while opening $rules. ($!)\n" ; }
+    %keys = pcd_get_keys() ;
+    unless ( %keys ) { print STDERR "pcd2html: Warning: No PCD inserted.\n" ; }
+    return %keys ;
 }
 
 #######################################################
@@ -352,6 +398,50 @@ sub GetDate {
   return $_ ;
 }
 
+#############################################################
+# Get strings for a homepage to go from main index
+# These strings have to be written in index.{deu,eng}
+# files as comment marked by the keywords "home" and "back"
+sub GetTextHome {
+
+  my ($back, $home);
+
+  open(TEXTFILE, "$_[0]" ) || die "Unable to open $_[0]\n" ;
+  while ( <TEXTFILE> ) {
+    if ( /^# *back: *(.*)/ ) {
+      $back = $1 ;
+    }
+    if ( /^# *home: *(.*)/ ) {
+      $home = $1 ;
+    }
+  }
+  close TEXTFILE ;
+  return ($back, $home);
+}
+
+#############################################################
+# For prevention of Spam mails you can provide an image
+# containing your e-mail adress in index.{eng,deu}
+sub GetSpamPrevention {
+
+  my $email = '';
+
+  open(TEXTFILE, "$_[0]" ) || return '' ;
+  while ( <TEXTFILE> ) {
+    if ( /^# *email: *(.*)/ ) {
+      $email = $1 ;
+      last ;
+    }
+  }
+  close TEXTFILE ;
+  if ( ! $email  ) { return '' ; }
+  $_ = $_[0] ;
+  # Add '../ ' if contained in beginning of path of $_[0]
+  if ( /^([.\/]+)/ ) { $email = $1 . $email ; }
+  if ( -s $email ) { return $email ; }
+  return '' ;
+}
+
 #############################################################
 # Get email from environment or build from loginname@host
 #
@@ -436,11 +526,51 @@ sub MyHtmlStart{
   print "</head>\n<body>\n" ;
 }
 
+###################################################################
+# Print top navigation
+#
+sub MyNavigation{
+  my $homename   = $_[0];
+  my $topname    = $_[1];
+  my $otherlang  = $_[2];
+  my $iconname   = $_[3];
+  my $othertitle = $_[4];
+  my $back       = $_[5];
+  my $home       = $_[6];
+  my $imgsrc     = $_[7];
+
+  print "<table class=\"navigation\" width=\"100%\">\n <tr>\n" ;
+  $imgsrc = '<img class="navigation" src="' . $imgsrc ;
+  if ( defined ( $back ) || defined ( $home ) ) {
+    print "  <td class=\"mainindex\">\n" ;
+    my $homestring = '   <a href="' . $home . '">' . $imgsrc . 'home.png" alt="' .
+                     $homename . "\"></a>\n" ;
+    if ( defined ( $back ) ) {
+      print '   <a href="'.$back.'">' . $imgsrc . 'pic_home.png" alt="' .
+            $topname . "\"></a>\n" ;
+      if ( defined ( $home ) ) {
+        print "  </td>\n  <td class=\"home\">\n" ;
+        print $homestring ;
+      }
+    } else {
+      print $homestring ;
+    }
+    print "  </td>\n" ;
+  }
+  print "  <td class=\"language\">\n" ;
+  print '   <a href="'.$otherlang.'">' . $imgsrc . $iconname.'" alt="' .
+        $othertitle . "\"></a>\n" ;
+  print "  </td>\n" ;
+  print " </tr>\n</table>\n" ;
+}
+
 ###################################################################
 # Print end of HTML information with subscription
 #
-sub MyHtmlEnd {
-  my ( $email, $using, $date, $author, $license );
+sub CreateFooterString {
+  my ( $email, $using, $date, $author, $license, $nospam, $end );
+
+  $footer_string = "" ;
 
   $_ = GetDate() ;
   /([\-\d]*).*/ ;
@@ -452,24 +582,48 @@ sub MyHtmlEnd {
     $license=$english{"license"} 
   }
   if ( defined $license && $license ne "" ) {
-    print "<p class=\"license\">\n$license\n</p>\n" ;
+    $footer_string = $footer_string . "<p class=\"license\">\n$license\n</p>\n" ;
   }  
-  print "<p class=\"footer\">\n" ;
+  $footer_string = $footer_string . "<p class=\"footer\">\n" ;
   if ( $_[0] eq "de" ) {
-    print " Diese Seite wurde am " ;
+    $footer_string = $footer_string . " Diese Seite wurde am " ;
     $_ = $date ;
     /(\d*)-(\d*)-(\d*)/ ;
-    print "$3.$2.$1 durch " ;
+    $footer_string = $footer_string . "$3.$2.$1 durch " ;
     $using = "mittels" ;
   } else {
-    print " This page was created on $date by " ;
+    $footer_string = $footer_string . " This page was created on $date by " ;
     $using = "using" ;
   }
-  $email  = GetEmail() ;
+
+  $end = " $using <a href=\"$pcd2html_source_url\">pcd2html</a>" ;
+  if ( $_[0] eq "de" ) { $end = $end . " erstellt" ; }
+  $end = $end . ".\n</p>\n</body>\n</html>\n" ;
   $author = GetAuthor() ;
-  print "<a href=\"mailto:$email\">$author</a> $using pcd2html" ;
-  if ( $_[0] eq "de" ) { print " erstellt" ; }
-  print ".\n</p>\n</body>\n</html>\n" ;
+  if ( ($nospam = GetSpamPrevention($_[1]) ) ) {
+    my ( $back, $home);
+    ($back, $home) = GetTextHome($_[1]) ;
+    if ( $home ) {
+      # if $home is a relative address obtain from $nospam the relative directory
+      if ( $home =~ /^([.\/]+)/ && $nospam =~ /^([.\/]+)/ ) { $home = $1 . $home ; }
+      $footer_string = $footer_string .
+                         "<a href=\"$PCD2HTMLMARKER$home\">$author</a> "
+    }
+    $footer_string = $footer_string . 
+                      '<a href="mailto:this.is@not.mymail.info"><img class="emil" src="' .
+                      "$PCD2HTMLMARKER$nospam\" alt=\"$author\"></a>" . $end ;
+  } else {
+    $footer_string = $footer_string . '<a href="mailto:' . GetEmail() . 
+                      "\">$author</a>" . $end ;
+  }
+}
+
+sub MyHtmlEnd {
+  my $subdir = '' ;
+  if ( $_[0] ) { $subdir = $_[0] ; }
+  $_ = $footer_string ;
+  s/$PCD2HTMLMARKER/$subdir/g ;
+  print ;
 }
 
 ###################################################################
@@ -649,16 +803,17 @@ sub GetKeyAndSource {
   my (  $pickey,  ## internal key of the cd in question 
         $picnum,  ## number of the picture
         $imgfile, ## *.img file for this image
-        $key, %dirs, $dir, $source
+        %dirs, $dir, $source, %keys, $pcddir
      );
 
   $pickey  = $_[0];
   $picnum  = $_[1];
   $imgfile = $_[2];
 
-  $key = pcd_get_key() ;
+  %keys   = pcd_get_keys() ;
+  $pcddir = $keys{$pickey} ;
 
-  if ( (!defined($key)) || ($pickey ne $key) ) {
+  unless ( $pcddir ) {
     %dirs = pcd_get_dirs() ;
     unless ( %dirs ) { 
       print STDERR "Can't obtain Kodak Photo-CD key \"$pickey\" or any valid image directory\n";
@@ -678,13 +833,12 @@ sub GetKeyAndSource {
       close (IMG) ;
       exit 0 ;
     }
-    $key = $pickey ;
   } else {
     unless ( $picnum =~ /extra/ ) {
-      $source = "$cdrom/photo_cd/images/img0" . $picnum . ".pcd";
+      $source = "$cdrom/$pcddir/images/img0" . $picnum . ".pcd";
     }
   }
-  return ($key, $source) ;
+  return ($pickey, $source) ;
 }
 
 #################################################################################
